
/*
 * Performance event assistance - Instruction events test driver
 */

/* 
 * Author:  P.J. Drongowski
 * Site:    sandsoftwaresound.net
 * Version: 1.1
 * Date:    14 January 2021
 *
 * Copyright (c) 2021 Paul J. Drongowski
 */

#include "pe_assist.h"
#include "pe_cortex_a72.h"

#define MSIZE    1000 

double matrix_a[MSIZE][MSIZE] ; // Left matrix operand
double matrix_b[MSIZE][MSIZE] ; // Right matrix operand
double matrix_r[MSIZE][MSIZE] ; // Matrix result

void initialize_matrices()
{
  // Define initial contents of the matrices
  for (int i = 0 ; i < MSIZE ; i++) {
    for (int j = 0 ; j < MSIZE ; j++) {
      matrix_a[i][j] = (float) rand() / RAND_MAX ;
      matrix_b[i][j] = (float) rand() / RAND_MAX ;
      matrix_r[i][j] = 0.0 ;
    }
  }
}

void multiply_matrices()
{
  // Multiply the two matrices
  for (int i = 0 ; i < MSIZE ; i++) {
    for (int j = 0 ; j < MSIZE ; j++) {
      float sum = 0.0 ;
      for (int k = 0 ; k < MSIZE ; k++) {
        sum = sum + matrix_a[i][k] * matrix_b[k][j] ;
    }
    matrix_r[i][j] = sum ;
    }
  }
}

void interchange()
{
  int i, j, k ;

    // Multiply the two matrices
    //
    // Please note that the nesting of the innermost
    // loops has been changed. The index variables j
    // and k change the most frequently and the access
    // pattern through the operand matrices is sequential
    // using a small stride (one.) This changes improves
    // access to memory data through the data cache. Data
    // translation lookaside buffer (DTLB) behavior is
    // also improved.
    for (i = 0 ; i < MSIZE ; i++) {
	for (k = 0 ; k < MSIZE ; k++) {
	    for (j = 0 ; j < MSIZE ; j++) {
		matrix_r[i][j] = matrix_r[i][j] +
		  (matrix_a[i][k] * matrix_b[k][j]) ;
	    }
        }
    }
}

/*
 * Test driver
 */
int main(int argc, char **argv)
{
  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Matrix Mult: Instruction events\n");
  fprintf(stdout, "Cortex-A72 native events\n");
  fprintf(stdout, "*************************************\n") ;

  // Measure events related to instructions and branches
  a72MeasureInstructionEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  initialize_matrices() ;
  multiply_matrices() ;

  // Stop the counters
  peStopCounting() ;

  // Print results

  a72PrintInstructionEvents(stdout) ;

  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Interchange: Instruction events\n");
  fprintf(stdout, "Cortex-A72 native events\n") ;
  fprintf(stdout, "*************************************\n") ;

  // Measure events related to instructions and branches
  a72MeasureInstructionEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  initialize_matrices() ;
  interchange() ;

  // Stop the counters
  peStopCounting() ;

  // Print results
  a72PrintInstructionEvents(stdout) ;

  return( 0 ) ;
}
