
/*
 * Walk a linear array using a stride
 */

/* 
 * Author:  P.J. Drongowski
 * Site:    sandsoftwaresound.net
 * Version: 1.0
 * Date:    12 February 2021
 *
 * Copyright (c) 2021 Paul J. Drongowski
 */

#include <stdio.h> 
#include <stdlib.h> 
#include <time.h> 

#include "pe_assist.h"
#include "pe_cortex_a72.h"

#define STRIDE 128
#define ARRAY_ELEMENTS (256*4194304)
char array[ARRAY_ELEMENTS] ;

/*
 * Test driver
 */
int main(int argc, char* argv[]) 
{
  register long int iterations = 1 ;
  register long long int sum = 0 ;
  register long int number_of_steps ;

  setlocale(LC_NUMERIC, "") ;

  if (argc > 1) {
    number_of_steps = atoi(argv[1]) ;
  } else {
    number_of_steps = (ARRAY_ELEMENTS / STRIDE) ;
  }

  iterations = 128 ;
  fprintf(stdout, "Number of array elements: %'ld\n", ARRAY_ELEMENTS) ;
  fprintf(stdout, "Stride:                   %'ld\n", STRIDE) ;
  fprintf(stdout, "Number of steps:          %'ld\n", number_of_steps) ;
  fprintf(stdout, "Number of iterations:     %'ld\n", iterations) ;
  
  a72MeasureDataAccessEvents() ;

  start_clock() ;
  peStartCounting() ;
  for ( ; iterations > 0 ; iterations--) {
    char *p = array ;
    for (int i = number_of_steps ; i > 0 ; i--) {
      sum = sum + *p ;
      p = p + STRIDE ;
    }
  }
  peStopCounting() ;

  print_clock_time(stdout, get_clock_time()) ;
  fprintf(stdout, "Sum: %lld\n", sum) ;
  a72PrintDataAccessEvents(stdout) ;

  return( 0 ) ; 
} 
