
/*
 * Performance event assistance functions - Matrix multiplication test
 */

/* 
 * Author:  P.J. Drongowski
 * Site:    sandsoftwaresound.net
 * Version: 1.1
 * Date:    14 January 2021
 *
 * Copyright (c) 2021 Paul J. Drongowski
 */

#include <stdlib.h>
#include <stdio.h>

#include "pe_assist.h"

#define ROWS     1000 
#define COLUMNS  1000

double matrix_a[ROWS][COLUMNS] ; // Left matrix operand
double matrix_b[ROWS][COLUMNS] ; // Right matrix operand
double matrix_r[ROWS][COLUMNS] ; // Matrix result

void initialize_matrices()
{
  // Define initial contents of the matrices
  for (int i = 0 ; i < ROWS ; i++) {
    for (int j = 0 ; j < COLUMNS ; j++) {
      matrix_a[i][j] = (float) rand() / RAND_MAX ;
      matrix_b[i][j] = (float) rand() / RAND_MAX ;
      matrix_r[i][j] = 0.0 ;
    }
  }
}

void multiply_matrices()
{
  // Multiply the two matrices
  for (int i = 0 ; i < ROWS ; i++) {
    for (int j = 0 ; j < COLUMNS ; j++) {
      float sum = 0.0 ;
      for (int k = 0 ; k < COLUMNS ; k++) {
        sum = sum + matrix_a[i][k] * matrix_b[k][j] ;
    }
    matrix_r[i][j] = sum ;
    }
  }
}

/*
 * Test driver
 */
int main(int argc, char **argv)
{
  // Measure events related to data read access
  peMeasureDataAccessEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  initialize_matrices() ;
  multiply_matrices() ;

  // Stop the counters
  peStopCounting() ;

  // Print results
  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Matrix Mult: Data access events\n") ;
  fprintf(stdout, "*************************************\n") ;

  pePrintDataAccessEvents(stdout) ;

    // Measure events related to fetch and branching
  peMeasureInstructionEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  initialize_matrices() ;
  multiply_matrices() ;

  // Stop the counters
  peStopCounting() ;

  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Matrix Mult: Instruction events\n") ;
  fprintf(stdout, "*************************************\n") ;

  // Print results
  pePrintInstructionEvents(stdout) ;
}
