
/*
 * Performance event assistance functions - Busy loop test driver
 */

/* 
 * Author:  P.J. Drongowski
 * Site:    sandsoftwaresound.net
 * Version: 1.1
 * Date:    14 January 2021
 *
 * Copyright (c) 2021 Paul J. Drongowski
 */

#include "pe_assist.h"

/*
 * Burn instructions and cycles.
 */
long long unsigned int spin_wheels(long long unsigned int iterations)
{
  long long unsigned int sum = 0 ;
  long long unsigned int i = 0 ;
  sum = 0 ;
  for (i = iterations ; i > 0 ; i--) {
    sum = sum + i ;
  }
  return( sum ) ;
}

/*
 * Test driver
 */
int main(int argc, char **argv)
{
  long long unsigned int sigma = 0 ;

  // Measure events related to fetch and branching
  peMeasureInstructionEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  sigma = spin_wheels(2000000000) ;
  printf("Sigma: %lld\n", sigma) ;

  // Stop the counters
  peStopCounting() ;

  // Print results
  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Busy loop: Instruction events\n") ;
  fprintf(stdout, "*************************************\n") ;
  
  pePrintInstructionEvents(stdout) ;

  // Measure events related to data read access
  peMeasureDataAccessEvents() ;
  
  // Clear and start the counters
  peStartCounting() ;

  // Do the workload
  sigma = spin_wheels(2000000000) ;
  printf("Sigma: %lld\n", sigma) ;

  // Stop the counters
  peStopCounting() ;

  // Print results
  fprintf(stdout, "*************************************\n") ;
  fprintf(stdout, "Busy loop: Data access events\n") ;
  fprintf(stdout, "*************************************\n") ;
  pePrintDataAccessEvents(stdout) ;
}
